{-----------------------------------------------------------------------------}
{ An import unit to expose the ToolHelp32 functions.                          }
{                                                                             }
{ NOTE!!!  This only works on Win95.  NT 3.5x does not provide these          }
{ functions in Kernel32.dll (or anywhere as far as I know).  I do not know    }
{ about NT 4.0, you are on your own there.                                    }
{                                                                             }
{ Copyright 1996, Brad Stowers.  All Rights Reserved.                         }
{ This unit can be freely used and distributed in commercial and private      }
{ environments, provied this notice is not modified in any way and there is   }
{ no charge for it other than nomial handling fees.  Contact me directly for  }
{ modifications to this agreement.                                            }
{-----------------------------------------------------------------------------}
{ Feel free to contact me if you have any questions, comments or suggestions  }
{ at bstowers@pobox.com or 72733,3374 on CompuServe.                          }
{ The lateset version will always be available on the web at:                 }
{   http://www.pobox.com/~bstowers/delphi/                                    }
{-----------------------------------------------------------------------------}
{ Date last modified:  10/19/96                                               }
{-----------------------------------------------------------------------------}

{ ----------------------------------------------------------------------------}
{ ToolHelp32 v1.00                                                            }
{ ----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  + Initial release.                                                   }
{          The comments in this unit are from the SDK.                        }
{ ----------------------------------------------------------------------------}

unit ToolHelp32;

{$IFNDEF WIN32}
  ERROR!  This unit only available on Win32!
{$ENDIF}

interface

uses Windows;

const
  MAX_MODULE_NAME32 = 255;

function CreateToolhelp32Snapshot(dwFlags, th32ProcessID: DWORD): THandle; stdcall;
//
// The th32ProcessID argument is only used if TH32CS_SNAPHEAPLIST or
// TH32CS_SNAPMODULE is specified. th32ProcessID == 0 means the current
// process.
//
// NOTE that all of the snapshots are global except for the heap and module
//	lists which are process specific. To enumerate the heap or module
//	state for all WIN32 processes call with TH32CS_SNAPALL and the
//	current process. Then for each process in the TH32CS_SNAPPROCESS
//	list that isn't the current process, do a call with just
//	TH32CS_SNAPHEAPLIST and/or TH32CS_SNAPMODULE.
//
// dwFlags
//
const
  TH32CS_SNAPHEAPLIST = $00000001;
  TH32CS_SNAPPROCESS  = $00000002;
  TH32CS_SNAPTHREAD   = $00000004;
  TH32CS_SNAPMODULE   = $00000008;
  TH32CS_SNAPALL	    = TH32CS_SNAPHEAPLIST or TH32CS_SNAPPROCESS or
                        TH32CS_SNAPTHREAD or TH32CS_SNAPMODULE;
  TH32CS_INHERIT	    = $80000000;
//
// Use CloseHandle to destroy the snapshot
//


(****** heap walking ***************************************************)

type
  PHEAPLIST32 = ^THEAPLIST32;
  THEAPLIST32 = packed record
    dwSize,
    th32ProcessID,    // owning process
    th32HeapID, 	    // heap (in owning process's context!)
    dwFlags: DWORD;
  end;

//
// dwFlags
//
const
  HF32_DEFAULT = 1;  // process's default heap
  HF32_SHARED  = 2;  // is shared heap

function Heap32ListFirst(hSnapshot: THandle; const lphl: THEAPLIST32): boolean; stdcall;
function Heap32ListNext(hSnapshot: THandle; const lphl: THEAPLIST32): boolean; stdcall;

type
  PHEAPENTRY32 = ^THEAPENTRY32;
  THEAPENTRY32 = packed record
    dwSize: DWORD;
    hHandle: THandle;	// Handle of this heap block
    dwAddress,      	// Linear address of start of block
    dwBlockSize,      // Size of block in bytes
    dwFlags,
    dwLockCount,
    dwResvd,
    th32ProcessID,    // owning process
    th32HeapID: DWORD;// heap block is in
  end;

//
// dwFlags
//
const
  LF32_FIXED    = $00000001;
  LF32_FREE     = $00000002;
  LF32_MOVEABLE = $00000004;

function Heap32First(const lphe: THEAPENTRY32; th32ProcessID, th32HeapID: DWORD): boolean; stdcall;
function Heap32Next(const lphe: THEAPENTRY32): boolean; stdcall;
function Toolhelp32ReadProcessMemory(th32ProcessID: DWORD;
                                     lpBaseAddress, lpBuffer: pointer;
                                     cbRead: DWORD;
                                     lpNumberOfBytesRead: PDWORD): boolean; stdcall;

(***** Process walking *************************************************)

type
  PPROCESSENTRY32 = ^TPROCESSENTRY32;
  TPROCESSENTRY32 = packed record
    dwSize,
    cntUsage,
    th32ProcessID,    	        // this process
    th32DefaultHeapID,
    th32ModuleID,     	        // associated exe
    cntThreads,
    th32ParentProcessID: DWORD; // this process's parent process
    pcPriClassBase: LongInt;    // Base priority of process's threads
    dwFlags: DWORD;
    szExeFile: array[0..MAX_PATH-1] of char;    // Path
  end;

const
  PCB_IDLE = 4;
  PCB_NORMAL = 8;
  PCB_HIGH = 13;
  PCB_REALTIME = 24;

function Process32First(hSnapshot: THandle; const lppe: TPROCESSENTRY32): boolean; stdcall;
function Process32Next(hSnapshot: THandle; const lppe: TPROCESSENTRY32): boolean; stdcall;

(***** Thread walking **************************************************)

type
  PTHREADENTRY32 = ^TTHREADENTRY32;
  TTHREADENTRY32 = packed record
    dwSize,
    cntUsage,
    th32ThreadID,	             // this thread
    th32OwnerProcessID: DWORD; // Process this thread is associated with
    tpBasePri,
    tpDeltaPri: LongInt;
    dwFlags: DWORD;
  end;

function Thread32First(hSnapshot: THandle; const lpte: TTHREADENTRY32): boolean; stdcall;
function Thread32Next(hSnapshot: THandle; const lpte: TTHREADENTRY32): boolean; stdcall;

(***** Module walking *************************************************)

type
  PMODULEENTRY32 = ^TMODULEENTRY32;
  TMODULEENTRY32 = packed record
    dwSize,
    th32ModuleID,	        // This module
    th32ProcessID,	        // owning process
    GlblcntUsage,	        // Global usage count on the module
    ProccntUsage: DWORD;	// Module usage count in th32ProcessID's context
    modBaseAddr: PBYTE; 	// Base address of module in th32ProcessID's context
    modBaseSize: DWORD;  	// Size in bytes of module starting at modBaseAddr
    hModule: HMODULE;  		// The hModule of this module in th32ProcessID's context
    szModule: array [0..MAX_MODULE_NAME32] of char;
    szExePath: array [0..MAX_PATH-1] of char;
  end;

//
// NOTE CAREFULLY that the modBaseAddr and hModule fields are valid ONLY
// in th32ProcessID's process context.
//

function Module32First(hSnapshot: THandle; const lpme: TMODULEENTRY32): boolean; stdcall;
function Module32Next(hSnapshot: THandle; const lpme: TMODULEENTRY32): boolean; stdcall;

implementation

function CreateToolhelp32Snapshot; external 'kernel32.dll' name 'CreateToolhelp32Snapshot';
function Heap32ListFirst; external 'kernel32.dll' name 'Heap32ListFirst';
function Heap32ListNext; external 'kernel32.dll' name 'Heap32ListNext';
function Heap32First; external 'kernel32.dll' name 'Heap32First';
function Heap32Next; external 'kernel32.dll' name 'Heap32Next';
function Toolhelp32ReadProcessMemory; external 'kernel32.dll' name 'Toolhelp32ReadProcessMemory';
function Process32First; external 'kernel32.dll' name 'Process32First';
function Process32Next; external 'kernel32.dll' name 'Process32Next';
function Thread32First; external 'kernel32.dll' name 'Thread32First';
function Thread32Next; external 'kernel32.dll' name 'Thread32Next';
function Module32First; external 'kernel32.dll' name 'Module32First';
function Module32Next; external 'kernel32.dll' name 'Module32Next';

end.

