unit SysAppInfo;

{-------------------------------------------------------------}
{                                                             }
{ Programmer:   Kevin S. Gallagher                            }
{ Version:      1.0.0                                         }
{ Description:                                                }
{ This form shows various pieces of information for a PC.     }
{   * System                                                  }
{   * Application                                             }
{   * Network                                                 }
{   * Language                                                }
{   * Directory                                               }
{   * Disk                                                    }
{   * Proccesses                                              }
{   * Window list                                             }
{ Several freeware components are used to retrieve the above  }
{ information (see credits below)                             }
{                                                             }
{ Version:  1.0.0                                             }
{ Requires:                                                   }
{ TVersionInfoResource component by Brad Stower. This is a    }
{ usually dropped onto a form and properties are set.         }
{ I use it w/o dropping it, and create it when needed.        }
{ Also, TProcList                                             }
{                                                             }
{ Credits                                       (alpha order) }
{ Grahame S Marsh                                             }
{ for lots of code used to retrieve various                   }
{ pieces of disk information.                                 }
{                                                             }
{ Peter.Plass                                                 }
{ Code to get loaded modules, and List of Windows             }
{                                                             }
{ Brad Stower                                                 }
{ A great deal of API stuff and components, and of course     }
{ his own components.                                         }
{                                                             }
{ NOTE: Although the supplied components are listed as        }
{ Freeware, there are some restricts. Make sure to read the   }
{ headers in the units and all read-me files.                 }
{                                                             }
{-------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, VersInfo, Registry, IniFiles, FindProc, ExtCtrls,
  FindWin;

type
  TfrmInformation = class(TForm)
    SysPageControl: TPageControl;
    SysTab: TTabSheet;
    AppTab: TTabSheet;
    cmdExit: TButton;
    NetTab: TTabSheet;
    LangTab: TTabSheet;
    DirTab: TTabSheet;
    MemTab: TTabSheet;
    MemView: TListView;
    lblCountUsage1: TLabel;
    lblPriority1: TLabel;
    lblPath1: TLabel;
    txtPath1: TEdit;
    lblCntUsed1: TLabel;
    lblPriority2: TLabel;
    Image1: TImage;
    ModView: TListView;
    Bevel1: TBevel;
    Image2: TImage;
    txtModPath: TEdit;
    Label1: TLabel;
    WinListTab: TTabSheet;
    ListBoxWinList: TListBox;
    cmdGetAllWinDoz: TButton;
    cmdGetAllVisibleWinDoz: TButton;
    cmdGetAllHiddenWinDoz: TButton;
    Image3: TImage;
    lblWinListPath: TLabel;
    GroupBox1: TGroupBox;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    lblClassName: TLabel;
    cmdReports: TButton;
    DiskTab: TTabSheet;
    DrivesCB: TComboBox;
    DriveImageBitmap: TImage;
    procedure cmdExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SysPageControlChange(Sender: TObject);
    procedure MemViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MemViewKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ModViewKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ModViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cmdGetAllWinDozClick(Sender: TObject);
    procedure cmdGetAllVisibleWinDozClick(Sender: TObject);
    procedure cmdGetAllHiddenWinDozClick(Sender: TObject);
    procedure MemViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ModViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBoxWinListKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBoxWinListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBoxWinListMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ListBoxWinListMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CheckBox3Click(Sender: TObject);
    procedure cmdReportsClick(Sender: TObject);
    procedure DrivesCBChange(Sender: TObject);
  private
    { Private declarations }
    WinList1: TWinList;
    ProcLister: TProcList;
    AppView: TListView;
    SysView: TListView;
    NetView: TListView;
    LangView: TListView;
    DiskView: TListView;
    DirView: TListView;
    DriveImage : integer;
    procedure PopulateWinDozList;
    procedure RefreshWinList;
    procedure RefreshWinListIcon;
    procedure PreProcessListWin;
    procedure ShowMemoryInfo(ClearList: boolean);
    procedure UpDateMemoryViews(Sender: TListView);
    procedure UpDateModViews(Sender: TListView);
    procedure PopulateNetSheet;
    procedure PopulateSysSheet;
    procedure PopulateLangSheet;
    procedure PopulateDirSheet;
    procedure PopulateDiskSheet;
    procedure FillDiskView;
    procedure PopulateAppSheet(Sender: TComponent);
    procedure ListViewsChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    function IsRegularKey(Shift: TShiftState):boolean;
    procedure FillInImage (Sender: TObject);
  public
    { Public declarations }
  end;

procedure ErrMsg(const S: String);
function StrCrLf(const cMsg:String):String;
function PadRight(const S: String; Len: Integer): String;

const
  C_UNKNOWN = '<unknown>';
  NO_VERSIONINFO = 'No version information found';

var
  frmInformation: TfrmInformation;
  VendorId      : string [12] = '<unknown>';
  CpuId         : byte = 0;
  ico           : TIcon;
  RefreshMem    : boolean;
  RefreshMod    : boolean;
  cPath         : String;
  DoFocusList   : boolean;

implementation

uses DiscUtil;

function StrCrLf(const cMsg:String):String;
var nLen,i:Integer;
begin
  nLen := Length(cMsg);
  i := 1;
  while i <= nLen do begin
    if cMsg[i] in [';','~'] then
      Result := Result + #13
    else
      Result := Result + cMsg[i];
    Inc(i);
  end;
end;

procedure ErrMsg(const S: String);
begin
  MessageDlg(StrCrLf(S),mtError,[mbOk],0);
end;

function strPadChR(const S: String; C: Char; Len: Integer): String;
begin
  Result := S;
  while Length(Result)<Len do Result := Result+C;
end;

function PadRight(const S: String; Len: Integer): String;
begin
  Result := strPadChR(S,#32,Len);
end;

procedure RealizeLength (var S : string);
begin
  SetLength(S,StrLen(PChar(S)))
end;

{$R *.DFM}

procedure TfrmInformation.FormCreate(Sender: TObject);
var
  C:TListColumn;
begin

  WinList1 := TWinList.Create(Self);
  ProcLister := TProcList.Create(Self);
  
  AppView := TListView.Create(Self);
  with AppView do begin
    Parent := AppTab;
    ViewStyle := vsReport;
    Align := alClient;
  end;

  SysView := TListView.Create(Self);
  with SysView do begin
    Parent := SysTab;
    ViewStyle := vsReport;
    Align := alClient;
    C := Columns.Add;
    C.Width := 170;
    C := Columns.Add;
    C.Width := 290;
    ColumnClick := False;
    BorderStyle := bsNone;
    Color := clSilver;
    ShowColumnHeaders := False;
    ReadOnly := True;
    OnChange := ListViewsChange;
  end;

  LangView := TListView.Create(Self);
  with LangView do begin
    Parent := LangTab;
    ViewStyle := vsReport;
    Align := alClient;
    C := Columns.Add;
    C.Width := 170;
    C := Columns.Add;
    C.Width := 290;
    ColumnClick := False;
    BorderStyle := bsNone;
    Color := clSilver;
    ShowColumnHeaders := False;
    ReadOnly := True;
    OnChange := ListViewsChange;
  end;

  DiskView := TListView.Create(Self);
  with DiskView do begin
    Parent := DiskTab;
    Top := SysPageControl.Top + 20;
    Width := DiskTab.Width - 2;
    Height := DiskTab.Height - 35;
    ViewStyle := vsReport;
    C := Columns.Add;
    C.Width := 135;
    C := Columns.Add;
    C.Width := 330;
    ColumnClick := False;
    BorderStyle := bsNone;
    Color := clSilver;
    ShowColumnHeaders := False;
    ReadOnly := True;
    OnChange := ListViewsChange;
  end;

  DirView := TListView.Create(Self);
  with DirView do begin
    Parent := DirTab;
    ViewStyle := vsReport;
    Align := alClient;
    C := Columns.Add;
    C.Width := 170;
    C := Columns.Add;
    C.Width := 290;
    ColumnClick := False;
    BorderStyle := bsNone;
    Color := clSilver;
    ShowColumnHeaders := False;
    ReadOnly := True;
    OnChange := ListViewsChange;
  end;

  NetView := TListView.Create(Self);
  with NetView do begin
    Parent := NetTab;
    ViewStyle := vsReport;
    Align := alClient;
    C := Columns.Add;
    C.Width := 93;
    C := Columns.Add;
    C.Width := 382;
    ColumnClick := False;
    BorderStyle := bsNone;
    Color := clSilver;
    ShowColumnHeaders := False;
    ReadOnly := True;
    OnChange := ListViewsChange;
  end;

  with MemView do begin
    ViewStyle := vsReport;
    C := Columns.Add;
    C.Width := 182;
    C.Caption := 'Process list';
    ColumnClick := False;
    RowSelect := True;
    Color := clWhite;
    ReadOnly := True;
  end;

  with ModView do begin
    ViewStyle := vsReport;
    C := Columns.Add;
    C.Width := 182;
    C.Caption := 'List of modules';
    ColumnClick := False;
    RowSelect := True;
    Color := clWhite;
    ReadOnly := True;
  end;

  SysPageControl.ActivePage := AppTab;
  SysPageControlChange(SysPageControl);

end;

procedure TfrmInformation.cmdExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmInformation.SysPageControlChange(Sender: TObject);
var
  T : TTabSheet;
  C : TPageControl;
begin

  C := (Sender as TPageControl);

  with C do begin
    T := ActivePage;
    if T = AppTab then begin
      if AppView.Items.Count = 0 then begin
        PopulateAppSheet(C);
        AppView.BorderStyle := bsNone;
        AppView.Color := clSilver;
        AppView.ShowColumnHeaders := False;
      end;
    end else if T = NetTab then begin
      if NetView.Items.Count = 0 then
        PopulateNetSheet;
    end else if T = DiskTab then begin
        PopulateDiskSheet;
    end else if T = SysTab then begin
      PopulateSysSheet;
    end else if T = LangTab then begin
      if LangView.Items.Count = 0 then
        PopulateLangSheet;
    end else if T = MemTab then begin
      ShowMemoryInfo(True);
      MemView.Selected := MemView.TopItem;
      MemView.SetFocus;
      UpDateMemoryViews(MemView);
    end else if T = WinListTab then begin
      DoFocusList := True;
      ListBoxWinList.Clear;
      PopulateWinDozList;
      try
        ListBoxWinList.SetFocus ;
      except
      end;
    end else if T = DirTab then begin
      if DirView.Items.Count = 0 then
        PopulateDirSheet;
      with DirView do begin
        Selected := FindCaption(0,'Current directory',False,True,True);
        SetFocus;
      end;
    end;
  end;
end;

procedure TfrmInformation.ListViewsChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  if Item.SubItems.Count > 0 then
    (Sender as TListView).RowSelect := True
  else
    (Sender as TListView).RowSelect := False;
end;

procedure TfrmInformation.PopulateDiskSheet;
begin
  GetLogicalDriveList(DrivesCB.Items);

  DrivesCB.ItemIndex := DrivesCB.Items.IndexOf(cPath[1] + ':\');
  FillDiskView;
end;

procedure TfrmInformation.FillDiskView;
  procedure Add(const ACaption, AValue : string);
  begin
    with DiskView.Items.Add do  begin
      Caption := ACaption;
      SubItems.Add (AValue);
    end;
  end;
var
  Drive : char;
  DriveType : TDriveType;
  Info : TDriveShellInfo;
  VolumeInfo : TVolumeInformation;
  DiscFreeSpace : TDiscFreeSpace;
begin
  DiskView.Items.Clear ;
  Drive := DrivesCB.Items[DrivesCB.ItemIndex][1];
  DriveType := GetDriveType(Drive);

  if DriveType = dtFloppy then begin
    DriveType := FloppyDriveSize(Drive);
    Screen.Cursor := crHourglass
  end;

  try
    Add('Drive type',DriveNames[DriveType]);
    GetDriveShellInfo(Drive, Info);
    DriveImage := Info.Image;
    FillInImage(Self);
    Add('ShellName',Info.DisplayName);

    while not GetVolumeInformation(Drive, VolumeInfo) do
      if not Retry (Drive) then Abort;

    with VolumeInfo do begin
      if DriveType = dtNetwork then
        Add('Volume label',GetNetworkVolumeName(Drive))
      else
        if VolumeName = '' then
          Add('Volume label','[none]')
        else
          Add('Volume label',VolumeName);

      Add('SerialNumber',FormatVolumeSerialNumber(VolumeSerialNumber));

      Add('Flags','');
      if FileSystemFlags and FS_CASE_IS_PRESERVED <> 0 then
        Add('','FS_CASE_IS_PRESERVED');

      if FileSystemFlags and FS_CASE_SENSITIVE <> 0 then
        Add('','FS_CASE_SENSITIVE');

      if FileSystemFlags and FS_UNICODE_STORED_ON_DISK <> 0 then
        Add('','FS_UNICODE_STORED_ON_DISK');

      if FileSystemFlags and FS_PERSISTENT_ACLS <> 0 then
        Add('','FS_PERSISTENT_ACLS');

      if FileSystemFlags and FS_VOL_IS_COMPRESSED <> 0 then
        Add('','FS_VOL_IS_COMPRESSED')
      else
        if FileSystemFlags and FS_FILE_COMPRESSION <> 0 then
          Add('','FS_FILE_COMPRESSION');

      Add('FileSystem',FileSystemName)
    end;

    while not GetDiscFreeSpace (Drive, DiscFreeSpace) do
      if not Retry (Drive) then Abort;

    with DiscFreeSpace do begin
      Add('SectorsPerCluster', IntToStr(SectorsPerCluster));
      Add('BytesPerSector', IntToStr(BytesPerSector));
      Add('FreeClusters', IntToStr(NumberOfFreeClusters));
      Add('Clusters',IntToStr(TotalNumberOfClusters));
      Add('FreeSpace', IntToStr(FreeSpace));
      Add('DiskSpace', IntToStr(TotalSpace));
    end;
    DiskView.Selected := DiskView.TopItem;
    DiskView.SetFocus;
  finally
    Screen.Cursor := crDefault
  end;
end;

procedure TfrmInformation.FillInImage(Sender: TObject);
var
  DriveBitmap : TBitmap;
begin
  DriveBitmap := TBitmap.Create;
  try
    if DriveImage > 0 then
      SmallImages.GetBitmap (DriveImage, DriveBitmap);
    DriveImageBitmap.Picture.Bitmap := DriveBitmap
  finally
    DriveBitmap.Free
  end;
end;

procedure TfrmInformation.PopulateDirSheet;
  procedure Add(const ACaption, AValue : string);
  begin
    with DirView.Items.Add do  begin
      Caption := ACaption;
      SubItems.Add (AValue);
    end;
  end;

  procedure AddFmt(const ACaption, AFormat : string; const Value: array of const );
  begin
    Add(ACaption, Format(AFormat, Value));
  end;
var
  S1,S2,S3: String;
begin
  SetLength(S1, MAX_PATH);
  GetWindowsDirectory(PChar (S1), MAX_PATH);
  RealizeLength(S1);
  Add('Windows directory', S1);

  SetLength(S1, MAX_PATH);
  GetSystemDirectory(PChar(S1), MAX_PATH);
  RealizeLength(S1);
  Add('Windows system directory', S1);

  SetLength(S1, MAX_PATH);
  GetCurrentDirectory(MAX_PATH, PChar (S1));
  RealizeLength(S1);
  Add('Current directory', S1);

  S1 := '';
  S2 := '';
  S3 := '';

  with TRegistry.Create do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    if OpenKey('Software\Microsoft\Windows\CurrentVersion', false) then
    begin
      try
        S1 := ReadString('ProgramFilesDir')
      except
        S1 := ''
      end;
      try
        S2 := ReadString('MediaPath')
      except
        S2 := ''
      end;
      try
        S3 := ReadString('CommonFilesDir')
      except
        S3 := ''
      end
    end;
  finally
    Free
  end;

  Add('Common files directory', S1);
  Add('Windows media directory', S2);
  Add('Program files directory', S3);

  AddFmt('Maximum path length', '%d', [MAX_PATH]);

end;

procedure TfrmInformation.PopulateLangSheet;
  procedure Add(const ACaption, AValue : string);
  begin
    with LangView.Items.Add do  begin
      Caption := ACaption;
      SubItems.Add (AValue);
    end;
  end;

  procedure AddFmt(const ACaption, AFormat : string; const Value: array of const );
  begin
    Add(ACaption, Format(AFormat, Value));
  end;

  procedure AddInstalled (const Caption : string; B : Bool);
  begin
    if B then
      Add(Caption, 'Installed')
    else
      Add(Caption, 'Not installed');
  end;

  procedure AddEnabled (const Caption : string; B : Bool);
  begin
    if B then
      Add(Caption, 'Enabled')
    else
      Add(Caption, 'Disabled');
  end;

var
  S: String;
begin

  SetLength(S,255);

  if SystemParametersInfo(SPI_LANGDRIVER, 0, @S, 0) then begin
    RealizeLength (S);
    Add('Language driver',S);
  end else
    Add('Language driver',C_UNKNOWN);

  AddInstalled('Hebrew/Arabic languages',
    Bool (GetSystemMetrics(SM_MIDEASTENABLED)));

  with SysLocale do begin
    AddEnabled('Far East', Bool (FarEast));
  end;

  Add('Currency string', CurrencyString);
  AddFmt('Currency format', '%d', [CurrencyFormat]);
  AddFmt('Negative currency format', '%d', [NegCurrFormat]);
  Add('Thousand separator', ThousandSeparator);
  Add('Decimal separator', DecimalSeparator);
  AddFmt('Currency decimals', '%d', [CurrencyDecimals]);
  Add('Date separator', DateSeparator);
  Add('Short date format', ShortDateFormat);
  Add('Long date format', LongDateFormat);
  Add('Time separator', TimeSeparator);
  Add('Time AM string', TimeAMString);
  Add('Time PM string', TimePMString);
  Add('Short time format', ShortTimeFormat);
  Add('Long time format', LongTimeFormat);

end;

procedure TfrmInformation.PopulateSysSheet;
  procedure GetCpu;
  asm
     mov     [CpuId], 4
     pushfd
     pop     eax
     mov     ecx, eax
     xor     eax, 0200000h
     push    eax
     popfd
     pushfd
     pop     eax
     xor     eax, ecx
     je      @@01
     mov     eax, 0
     db      0fh
     db      0a2h
     mov     [byte ptr VendorId], 12
     mov     [dword ptr VendorId+1], ebx
     mov     [dword ptr VendorId+5], edx
     mov     [dword ptr VendorId+9], ecx
     mov     eax, 1
     db      0fh
     db      0a2h
     shr     eax, 8
     and     al, 0fh
     mov     [CpuId], al
  @@01:
  end;

  procedure Add(const ACaption, AValue : string);
  begin
    with SysView.Items.Add do begin
      Caption := ACaption;
      SubItems.Add(AValue)
    end
  end;

  procedure AddFmt(const ACaption, AFormat : string; const Value: array of const );
  begin
    Add(ACaption,Format(AFormat, Value))
  end;

  procedure AddInt(const ACaption, AFormat : string; Value: Extended);
  begin
    with SysView.Items.Add do begin
      Caption := ACaption;
      if Value > 0 then
        SubItems.Add(FormatFloat(AFormat, Value ))
      else SubItems.Add('0');
    end
  end;

var
  MemoryStatus: TMemoryStatus;
  SystemInfo  : TSystemInfo;
  DC          : hDC;
  I: Integer;
begin
  SysView.Items.Clear ;

  with SysView.Items.Add do begin
    Caption := 'Memory';
  end;

  MemoryStatus.dwLength := SizeOf (TMemoryStatus);
  GlobalMemoryStatus (MemoryStatus);

  with MemoryStatus do begin
    AddInt('Total physical', '#Kb',   dwTotalPhys DIV 1024);
    AddInt('Free physical', '#Kb',    dwAvailPhys DIV 1024);
    AddInt('Total virtual', '#Kb',    dwTotalVirtual DIV 1024);
    AddInt('Available virtual', '#Kb',dwAvailVirtual DIV 1024);
  end;

  with SysView.Items.Add do begin
    Caption := 'Processor';
  end;

  GetCPU;
  Add('Vendor', VendorID);
  {AddFmt('Id', '%d', [CpuId]);                     KSG this does work }
  GetSystemInfo(SystemInfo);

  with SystemInfo do
  begin
    AddFmt('Type', '%d', [dwProcessorType]);
    {AddFmt('Level', '%d', [wProcessorLevel]);
    AddFmt('Revision', '%x', [wProcessorRevision])  KSG this does work }
  end;

  with SysView.Items.Add do begin
    Caption := 'Keyboard';
  end;

  with TIniFile.Create ('system.ini') do
  try
    Add('Driver description',ReadString ('boot.description','keyboard.typ',C_UNKNOWN));
  finally
    Free;
  end;

  if SystemParametersInfo (SPI_GETKEYBOARDDELAY, 0, @I, 0) then
    AddFmt('Keyboard delay', '%d', [I])
  else
    Add('Keyboard delay',C_UNKNOWN);

  if SystemParametersInfo (SPI_GETKEYBOARDSPEED, 0, @I, 0) then
    AddFmt('Keyboard repeat speed', '%d', [I])
  else
    Add('Keyboard repeat speed',C_UNKNOWN);

  with SysView.Items.Add do begin
    Caption := 'Mouse';
  end;

  with TIniFile.Create ('system.ini') do
  try
    Add('Description', ReadString ('boot.description','mouse.drv',C_UNKNOWN));
  finally
    Free
  end;
  if Bool (GetSystemMetrics (SM_MOUSEPRESENT)) then
    Add('Status', 'Present')
  else
    Add('Status', 'Missing');

  if Bool (GetSystemMetrics (SM_SWAPBUTTON)) then
    Add('Buttons', 'Swapped')
  else
    Add('Buttons', 'Normal');

  with SysView.Items.Add do begin
    Caption := 'Screen';
  end;

  DC := GetDC (0);
  try
    AddFmt('Driver version', '%d', [GetDeviceCaps (DC, DRIVERVERSION)]);
    AddFmt('Screen size', '%d, %d mm', [GetDeviceCaps (DC, HORZSIZE), GetDeviceCaps (DC, VERTSIZE)]);
    AddFmt('Screen resolution', '%d, %d pixels', [GetDeviceCaps (DC, HORZRES), GetDeviceCaps (DC, VERTRES)]);
  finally
    ReleaseDC (DC, 0)
  end;
end;

procedure TfrmInformation.PopulateAppSheet(Sender: TComponent);
begin
  with TVersionInfoResource.Create(Sender) do begin
    VersionListView := AppView;
    Filename := Application.ExeName;
  end;

  if AppView.Items.Count = 0 then
    with AppView.Items.Add do Caption := NO_VERSIONINFO;
end;

{ More information could be added if so desired }
procedure TfrmInformation.PopulateNetSheet;
const
  C1 = '<unavailable>';
var
  S: String;
  I: dword;
begin

  SetLength(S, MAX_COMPUTERNAME_LENGTH + 1);
  I := MAX_COMPUTERNAME_LENGTH + 1;

  GetComputerName(PChar(S), I);
  RealizeLength(S);

  with NetView.Items.Add do begin
    Caption := 'Computer name';
    SubItems.Add (S)
  end;

  with NetView.Items.Add do begin
    Caption := 'Network';
    if bool (GetSystemMetrics (SM_NETWORK)) then
      SubItems.Add ('Installed')
    else
      SubItems.Add ('Not installed');
  end;

  with NetView.Items.Add do begin
    with TIniFile.Create ('system.ini') do
      try
        Caption := 'Description';
        SubItems.Add(ReadString('boot.description','network.drv', C1));
      finally
        Free
      end
  end;
end;

{ Used to load ListView (left side) on "Processes" tabsheet }
procedure TfrmInformation.ShowMemoryInfo(ClearList: boolean);
var
  i,n:Integer;
begin

  if not ClearList then exit;

  Screen.Cursor := crHourGlass;
  try
    MemView.Items.Clear;
    ModView.Items.Clear;
    with ProcLister do begin
      GetProcessInfo;
      n := ProcessList.Count;
      for i := 0 to n -1 do
        with MemView.Items.Add do   Caption := ProcessList[i];
    end;
  finally
    Screen.Cursor := crDefault ;
  end;

end;

{ Used to load ListView (right side) on "Processes" tabsheet.
  This is tiggered by mouse/keyboard events for left-side ListView  }
procedure TfrmInformation.UpDateMemoryViews(Sender: TListView);
var
  iItem: Integer;
  i,n:Integer;
begin

  if Sender.SelCount = 0 then
    exit;

  ModView.Items.Clear;

  iItem := Sender.Selected.Index;

  ico := TIcon.Create;

  try
    ico.Handle := ProcLister.GetP_IconHandle(iItem);
    Screen.Cursor := crHourGlass;
    with ProcLister do begin
      lblCntUsed1.Caption := IntToStr(GetP_CountUsage(iItem));
      lblPriority2.Caption := IntToStr(GetP_Priority(iItem));

      txtPath1.Text := GetP_ExePath(iItem);
      GetModuleInfo(GetP_ID( Sender.Selected.Index ));

      n := ModuleList.Count;

      for i := 0 to n -1 do
        with ModView.Items.Add do
          Caption := ModuleList[i];
    end;

    Image1.Picture.Icon := ico;
    txtPath1.Hint := txtPath1.Text;

  except
    Screen.Cursor := crDefault;
    ErrMsg('Unforeseen problems in~UpDateMemoryViews');
  end;
  Screen.Cursor := crDefault;
end;

procedure TfrmInformation.MemViewMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  UpDateMemoryViews(TListView(Sender));
end;

procedure TfrmInformation.MemViewKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if RefreshMem then  UpDateMemoryViews(TListView(Sender));
end;

procedure TfrmInformation.UpDateModViews(Sender: TListView);
var
  iItem: Integer;
begin

  if Sender.SelCount = 0 then
    exit;

  iItem := Sender.Selected.Index;

  ico := TIcon.Create;
  ico.Handle := ProcLister.GetM_IconHandle(iItem);
  Image2.Picture.Icon := ico;

  txtModPath.Text := ProcLister.GetM_szExePath(iItem);
  txtModPath.Hint := txtModPath.Text;

end;

function TfrmInformation.IsRegularKey(Shift: TShiftState):boolean;
begin
  Result := not((Shift = [ssAlt]) or
               (Shift = [ssCtrl]) or
               (Shift = [ssShift]));
end;

procedure TfrmInformation.ModViewKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Caption := TListView(Sender).Name;
  if not (Shift = [ssShift]) then
    UpDateModViews(TListView(Sender));
end;

procedure TfrmInformation.ModViewMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  UpDateModViews(TListView(Sender));
end;

procedure TfrmInformation.PreProcessListWin;
begin
  WinList1.TaskBarSty := Checkbox2.checked;

  if Checkbox2.Checked then begin
    Checkbox1.Checked := False;
    Checkbox1.Enabled := False;
  end else
    Checkbox1.Enabled := True;
  WinList1.EmptyNames:= Checkbox1.Checked;
end;

procedure TfrmInformation.PopulateWinDozList;
begin
  PreProcessListWin;
  WinList1.WhatDoFind := id_all;
  RefreshWinList;
end;

procedure TfrmInformation.cmdGetAllWinDozClick(Sender: TObject);
begin
  PreProcessListWin;
  WinList1.WhatDoFind := id_all;
  RefreshWinList;
  if DoFocusList then
    ListBoxWinList.SetFocus ;
end;

procedure TfrmInformation.cmdGetAllVisibleWinDozClick(Sender: TObject);
begin
  PreProcessListWin;
  WinList1.WhatDoFind := id_visonly;
  RefreshWinList;
  if DoFocusList then
    ListBoxWinList.SetFocus ;
end;

procedure TfrmInformation.cmdGetAllHiddenWinDozClick(Sender: TObject);
begin
  PreProcessListWin;
  WinList1.WhatDoFind := id_invisonly;
  RefreshWinList;
  if DoFocusList then
    ListBoxWinList.SetFocus ;
end;

procedure TfrmInformation.RefreshWinList;
begin
  WinList1.Execute;
  ListBoxWinList.Items.Assign(WinList1.WindowList);
  ListBoxWinList.ItemIndex := 0;
  RefreshWinListIcon;
end;
procedure TfrmInformation.MemViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  RefreshMem := IsRegularKey(Shift);
end;

procedure TfrmInformation.ModViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  RefreshMod := IsRegularKey(Shift);
end;

procedure TfrmInformation.ListBoxWinListKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  RefreshWinListIcon;
end;

procedure TfrmInformation.ListBoxWinListKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  RefreshWinListIcon;
end;

procedure TfrmInformation.RefreshWinListIcon;
var
  iItem:Integer;
begin

  try
    if ListBoxWinList.Items.count < 1
      then exit;

    iItem := ListBoxWinList.ItemIndex;
    if iItem  < 0
      then exit;

    ico := TIcon.Create;
    ico.Handle := WinList1.GetIconHandle(iItem);
    Image3.Picture.Icon := ico;

    lblWinListPath.Caption := WinList1.GetWinExePath(iItem);
    lblClassName.Caption:= 'ClassName: ' + WinList1.GetWnClasName(iItem);

  except
    ErrMsg('Problem occured in~RefreshWinListIcon');
  end;

end;

procedure TfrmInformation.ListBoxWinListMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  RefreshWinListIcon;
end;

procedure TfrmInformation.ListBoxWinListMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  RefreshWinListIcon;
end;

procedure TfrmInformation.CheckBox3Click(Sender: TObject);
begin
 WinList1.EnableTimer := CheckBox3.checked;
end;

{ Sends info from each TabSheet to a ASCII file. If the file
  already exist it gets appended, else it is created }
procedure TfrmInformation.cmdReportsClick(Sender: TObject);
const
  NPAD = 25;
  CTAB = '    ';
var
  Proc1: TProcList;
  nItems,
  y,i,n,x: Integer;
  F: TextFile;
  S: String;
  cFile: String;
begin

  with TSaveDialog.Create(self) do begin
    InitialDir := cPath;
    if Execute then
      cFile := FileName
    else
      exit;
  end;

  if ExtractFileExt(cFile) = '' then
    cFile := cFile + '.txt';

  DoFocusList := False;
  AssignFile(F, cFile);
  if FileExists(cFile) then
    Append(F)
  else
    ReWrite(F);

  Screen.Cursor := crHourglass;
  
  try
    cmdReports.Enabled := False;
    WriteLn(F,'');
    WriteLn(F,'Generated: ' + DateToStr(Date) + '  ' + TimeToStr(Time));

    nItems := AppView.Items.Count;

    WriteLn(F,'Application information');

    if nItems > 1 then begin
      for i := 0 to nItems -1 do begin
        S := PadRight(AppView.Items.Item[i].Caption,NPAD);
        if AppView.Items.Item[i].SubItems.Count >0 then begin
          S := S +  ' - ' + AppView.Items.Item[i].SubItems[0];
          WriteLn(F,CTAB + S);
        end else
          WriteLn(F,S);
      end;
    end else
      WriteLn(F,CTAB + NO_VERSIONINFO);

    WriteLn(F,'');

    nItems := SysView.Items.Count;

    for i := 0 to nItems -1 do begin
      S := PadRight(SysView.Items.Item[i].Caption,NPAD);
      if SysView.Items.Item[i].SubItems.Count >0 then begin
        S := S +  ' - ' + SysView.Items.Item[i].SubItems[0];
        WriteLn(F,CTAB + S);
      end else
        WriteLn(F,S);
    end;

    WriteLn(F,'');

    WriteLn(F,'Network information');

    if NetView.Items.Count = 0 then
      PopulateNetSheet;
    nItems := NetView.Items.Count;
    with NetView,Items do begin
      for i := 0 to nItems -1 do begin
        S := PadRight(Item[i].Caption,NPAD);
        if Item[i].SubItems.Count >0 then begin
          S := S +  ' - ' + Item[i].SubItems[0];
          WriteLn(F,CTAB + S);
        end else
          WriteLn(F,S);
      end;
    end;

    WriteLn(F,'');

    WriteLn(F,'Language information');

    if LangView.Items.Count = 0 then
      PopulateLangSheet;
    nItems := LangView.Items.Count;
    with LangView,Items do begin
      for i := 0 to nItems -1 do begin
        S := PadRight(Item[i].Caption,NPAD);
        if Item[i].SubItems.Count >0 then begin
          S := S +  ' - ' + Item[i].SubItems[0];
          WriteLn(F,CTAB + S);
        end else
          WriteLn(F,S);
      end;
    end;

    WriteLn(F,'');

    WriteLn(F,'Disk information');

    if DirView.Items.Count = 0 then
      PopulateDirSheet;
    nItems := DirView.Items.Count;
    with DirView,Items do begin
      for i := 0 to nItems -1 do begin
        S := PadRight(Item[i].Caption,NPAD);
        if Item[i].SubItems.Count >0 then begin
          S := S +  ' - ' + Item[i].SubItems[0];
          WriteLn(F,CTAB + S);
        end else
          WriteLn(F,S);
      end;
    end;

    ShowMemoryInfo(True);
    MemView.Selected := MemView.TopItem;

    UpDateMemoryViews(MemView);

    WriteLn(F,'');

    WriteLn(F,'Processess information');

    Screen.Cursor := crHourGlass;

    Proc1 := TProcList.Create(Self);
    try
      with Proc1 do begin
        GetProcessInfo;
        n := ProcessList.Count;

        for i := 0 to n -1 do begin
          WriteLn(F,ProcessList[i]);
          GetModuleInfo(GetP_ID(i));
          x := ModuleList.Count;
          for y := 0 to x -1 do
            WriteLn(F,CTAB + ModuleList[y]);
        end;
      end;
    finally
      Proc1.Destroy;
      Screen.Cursor := crDefault ;
    end;

    {----------------------------------------------------------------
      This section does the Window list, and can get messy real
      fast. I decided rather then spend a load of time to leave
      it simple. If you want to do more, then you can invoke
      each button on the Window List tab-sheet, and cycle through
      after calling the Click events to write each window listing

      No padding is done since some paths can be very long which
      makes for difficult reading and printing. Something to do
      on another day...
    ----------------------------------------------------------------}
    WriteLn(F,'');
    WriteLn(F,'Processess information');

    if ListBoxWinList.Items.Count > 0 then
      ListBoxWinList.Clear;

    PopulateWinDozList;

    for i := 0 to ListBoxWinList.Items.Count -1 do
      WriteLn(F,CTAB + ListBoxWinList.Items[i] + CTAB +
        WinList1.GetWinExePath(i));

    WriteLn(F,'');

    { I think this is up to the developer how to code.
      There are many things to consider like will all
      drives be ready, do we want, whats the best
      way to format the report. Also do we get all
      drives?
    }
    WriteLn(F,'Disk information not coded yet')
  finally
    CloseFile(F);
    Screen.Cursor := crDefault;
    cmdReports.Enabled := True;
  end;
end;

procedure TfrmInformation.DrivesCBChange(Sender: TObject);
begin
  FillDiskView;
end;

initialization
  RefreshMem := False;
  RefreshMod := False;
  cPath := ExtractFilePath(Application.ExeName);;
end.
