unit TreeForm;
// NOTE: I have bookmarked code to generalize
{-------------------------------------------------------------}
{                                                             }
{ Programmer:   Kevin S. Gallagher                            }
{ Version:      1.0.0                                         }
{ Description:                                                }
{ This form shows various pieces of information for a PC.     }
{   * System                                                  }
{   * Application                                             }
{   * Network                                                 }
{   * Language                                                }
{   * Directory                                               }
{   * Disk                                                    }
{   * Proccesses                                              }
{ It is a alternate to the form TfrmInformation which used a  }
{ PageControl with ListViews and ListBoxes to show system     }
{ information. I should point out that TfrmInformation shows  }
{ more information.                                           }
{                                                             }
{ Several freeware components are used to retrieve the above  }
{ information (see credits below)                             }
{                                                             }
{ Requires:                                                   }
{ TVersionInfoResource component by Brad Stower. This is a    }
{ usually dropped onto a form and properties are set.         }
{                                                             }
{ Credits                                       (alpha order) }
{ Grahame S Marsh                                             }
{ for lots of code used to retrieve various                   }
{ pieces of disk information.                                 }
{                                                             }
{ Peter.Plass                                                 }
{ Code to get loaded modules, and List of Windows             }
{                                                             }
{ Brad Stower                                                 }
{ A great deal of API stuff and components, and of course     }
{ his own components.                                         }
{                                                             }
{ NOTE: Although the supplied components are listed as        }
{ Freeware, there are some restricts. Make sure to read the   }
{ headers in the units and all read-me files and legal.txt    }
{                                                             }
{-------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, VersInfo, ExtCtrls,  Registry, IniFiles,
  FindProc, StdCtrls, Buttons, Printers, ItemProp;

type
  TfrmSysTree = class(TForm)
    TreeView1: TTreeView;
    ListView1: TListView;
    Splitter1: TSplitter;
    ToolPanel: TPanel;
    DrivesCB: TComboBox;
    StatusPanel: TPanel;
    cmdExit: TButton;
    TempPanel: TPanel;
    lblOnTempPanel: TLabel;
    cmdWhatsThisHelp: TSpeedButton;
    cmdPrintInformation: TSpeedButton;
    cmdDiskReport: TSpeedButton;
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure FormResize(Sender: TObject);
    procedure DrivesCBChange(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure TreeView1DblClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ListView1Enter(Sender: TObject);
    procedure cmdPrintInformationClick(Sender: TObject);
    procedure cmdDiskReportClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    ProcLister: TProcList;
    ProcPaths: TStringList;
    procedure PopulateAppNode;
    procedure PopulateSysNode;
    procedure PopulateDirNode;
    procedure PopulateNetNode;
    procedure PopulateLangNode;
    procedure PopulateDiskNode;
    procedure PopulateAppsRunNode;
    procedure PopulateModemNode;
    procedure PopulatePrintNode;
    procedure FillDiskView;
    procedure ShowAboutScreen;
    procedure ClearListViewHeaders;
  public
    { Public declarations }
  end;

const
  C_UNKNOWN = '<unknown>';
  NO_VERSIONINFO = 'No version information found';
  STATUS_WAIT_MESSAGE = 'Gathering information...';

var
  frmSysTree: TfrmSysTree;
  VendorId      : string [12] = '<unknown>';
  CpuId         : byte = 0;
  cPath         : String;

implementation

uses DiscUtil;

{$R *.DFM}

procedure RealizeLength (var S : string);
begin
  SetLength(S,StrLen(PChar(S)))
end;

procedure TfrmSysTree.ClearListViewHeaders;
begin
  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Caption := '';
    Columns.Items[1].Caption := '';
  end;
end;

procedure TfrmSysTree.FormActivate(Sender: TObject);
var
  TN,T:TTreeNode;
  iMarker:Integer;
begin
  cPath := ExtractFilePath(Application.ExeName);;

  Refresh;
  iMarker := 1;
  with TreeView1,Items do begin
    TN := Add(TreeView1.Selected ,'System Information');

    AddChildObject(TN,'Application',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'System',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Directories',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Disk',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Network',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Language',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Applications running',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Modem',Pointer(iMarker));

    Inc(iMarker);
    AddChildObject(TN,'Printer',Pointer(iMarker));
    iMarker := 20;
    AddObject(TN,'About',Pointer(iMarker));

    TN.Expand(False);

    T := TopItem.GetFirstChild ;

    if T <> nil then
      T.Selected := True;
  end;

  if TempPanel <> nil then  TempPanel.Free;
end;

procedure TfrmSysTree.TreeView1Change(Sender: TObject; Node: TTreeNode);
var
  x:Integer;
begin
  DrivesCB.Visible := False;

  if (Node.Level = 0) and (Node.Index = 0) then
    exit;

  x := Integer(Node.Data);

  if x <> 1 then begin
    with ListView1 do begin
      Items.Clear ;
      if Columns.Count = 0 then begin
        Columns.Add;
        Columns.Add;
      end;
    end;
  end;

  case x of
    1: PopulateAppNode;
    2: PopulateSysNode;
    3: PopulateDirNode;
    4: PopulateDiskNode;
    5: PopulateNetNode;
    6: PopulateLangNode;
    7: PopulateAppsRunNode;
    8: PopulateModemNode;
    9: PopulatePrintNode;
  else
    ClearListViewHeaders;
  end;
end;

procedure TfrmSysTree.PopulateAppNode;
begin
  StatusPanel.Caption := STATUS_WAIT_MESSAGE;
  StatusPanel.Refresh;

  with TVersionInfoResource.Create(Self) do begin
    VersionListView := ListView1;
    Filename := Application.ExeName;
  end;

  StatusPanel.Caption := '';
end;

procedure TfrmSysTree.PopulateDirNode;
  procedure Add(const ACaption, AValue : string);
  begin
    with ListView1.Items.Add do  begin
      Caption := ACaption;
      SubItems.Add (AValue);
    end;
  end;

  procedure AddFmt(const ACaption, AFormat : string; const Value: array of const );
  begin
    Add(ACaption, Format(AFormat, Value));
  end;
var
  S1,S2,S3: String;
begin

  StatusPanel.Caption := STATUS_WAIT_MESSAGE;
  StatusPanel.Refresh;

  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Width := 130;    
    Columns.Items[0].Caption := 'Items';
    Columns.Items[1].Caption := 'Value';
  end;

  SetLength(S1, MAX_PATH);
  GetWindowsDirectory(PChar (S1), MAX_PATH);
  RealizeLength(S1);
  Add('Windows directory', S1);

  SetLength(S1, MAX_PATH);
  GetSystemDirectory(PChar(S1), MAX_PATH);
  RealizeLength(S1);
  Add('Windows system directory', S1);

  SetLength(S1, MAX_PATH);
  GetCurrentDirectory(MAX_PATH, PChar(S1));
  RealizeLength(S1);
  Add('Current directory', S1);

  S1 := '';
  S2 := '';
  S3 := '';

  with TRegistry.Create do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    if OpenKey('Software\Microsoft\Windows\CurrentVersion', false) then
    begin
      try
        S1 := ReadString('ProgramFilesDir')
      except
        S1 := ''
      end;
      try
        S2 := ReadString('MediaPath')
      except
        S2 := ''
      end;
      try
        S3 := ReadString('CommonFilesDir')
      except
        S3 := ''
      end
    end;
  finally
    Free
  end;

  Add('Common files directory', S1);
  Add('Windows media directory', S2);
  Add('Program files directory', S3);

  AddFmt('Maximum path length', '%d', [MAX_PATH]);

  StatusPanel.Caption := '';
end;

procedure TfrmSysTree.PopulateSysNode;
  procedure GetCpu;
  asm
     mov     [CpuId], 4
     pushfd
     pop     eax
     mov     ecx, eax
     xor     eax, 0200000h
     push    eax
     popfd
     pushfd
     pop     eax
     xor     eax, ecx
     je      @@01
     mov     eax, 0
     db      0fh
     db      0a2h
     mov     [byte ptr VendorId], 12
     mov     [dword ptr VendorId+1], ebx
     mov     [dword ptr VendorId+5], edx
     mov     [dword ptr VendorId+9], ecx
     mov     eax, 1
     db      0fh
     db      0a2h
     shr     eax, 8
     and     al, 0fh
     mov     [CpuId], al
  @@01:
  end;

  procedure Add(const ACaption, AValue : string);
  begin
    with ListView1.Items.Add do begin
      Caption := ACaption;
      SubItems.Add(AValue)
    end
  end;

  procedure AddFmt(const ACaption, AFormat : string; const Value: array of const );
  begin
    Add(ACaption,Format(AFormat, Value))
  end;

  procedure AddInt(const ACaption, AFormat : string; Value: Extended);
  begin
    with ListView1.Items.Add do begin
      Caption := ACaption;
      if Value > 0 then
        SubItems.Add(FormatFloat(AFormat, Value ))
      else SubItems.Add('0');
    end
  end;

var
  MemoryStatus: TMemoryStatus;
  SystemInfo  : TSystemInfo;
  DC          : hDC;
  I: Integer;
begin

  StatusPanel.Caption := STATUS_WAIT_MESSAGE;
  StatusPanel.Refresh;
  
  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Caption := 'Items';
    Columns.Items[0].Width := 130;
    Columns.Items[1].Caption := 'Value';
    Columns.Items[1].Width := 270;
    with Items.Add do begin
      Caption := 'Memory';
    end;
  end;

  MemoryStatus.dwLength := SizeOf (TMemoryStatus);
  GlobalMemoryStatus (MemoryStatus);

  with MemoryStatus do begin
    AddInt('Total physical', '#Kb',   dwTotalPhys DIV 1024);
    AddInt('Free physical', '#Kb',    dwAvailPhys DIV 1024);
    AddInt('Total virtual', '#Kb',    dwTotalVirtual DIV 1024);
    AddInt('Available virtual', '#Kb',dwAvailVirtual DIV 1024);
  end;

  with ListView1.Items.Add do begin
    Caption := 'Processor';
  end;

  GetCPU;
  Add('Vendor', VendorID);
  AddFmt('Id', '%d', [CpuId]);
  GetSystemInfo(SystemInfo);

  with SystemInfo do
  begin
    AddFmt('Type', '%d', [dwProcessorType]);
    AddFmt('Level', '%d', [wProcessorLevel]);
    AddFmt('Revision', '%x', [wProcessorRevision]);
  end;

  with ListView1.Items.Add do begin
    Caption := 'Keyboard';
  end;

  with TIniFile.Create ('system.ini') do
  try
    Add('Driver description',ReadString ('boot.description','keyboard.typ',C_UNKNOWN));
  finally
    Free;
  end;

  if SystemParametersInfo (SPI_GETKEYBOARDDELAY, 0, @I, 0) then
    AddFmt('Keyboard delay', '%d', [I])
  else
    Add('Keyboard delay',C_UNKNOWN);

  if SystemParametersInfo (SPI_GETKEYBOARDSPEED, 0, @I, 0) then
    AddFmt('Keyboard repeat speed', '%d', [I])
  else
    Add('Keyboard repeat speed',C_UNKNOWN);

  with ListView1.Items.Add do begin
    Caption := 'Mouse';
  end;

  with TIniFile.Create ('system.ini') do
  try
    Add('Description', ReadString ('boot.description','mouse.drv',C_UNKNOWN));
  finally
    Free
  end;
  if Bool (GetSystemMetrics (SM_MOUSEPRESENT)) then
    Add('Status', 'Present')
  else
    Add('Status', 'Missing');

  if Bool (GetSystemMetrics (SM_SWAPBUTTON)) then
    Add('Buttons', 'Swapped')
  else
    Add('Buttons', 'Normal');

  with ListView1.Items.Add do begin
    Caption := 'Screen';
  end;

  DC := GetDC (0);
  try
    AddFmt('Driver version', '%d', [GetDeviceCaps (DC, DRIVERVERSION)]);
    AddFmt('Screen size', '%d, %d mm', [GetDeviceCaps (DC, HORZSIZE), GetDeviceCaps (DC, VERTSIZE)]);
    AddFmt('Screen resolution', '%d, %d pixels', [GetDeviceCaps (DC, HORZRES), GetDeviceCaps (DC, VERTRES)]);
  finally
    ReleaseDC (DC, 0)
  end;

  StatusPanel.Caption := '';

end;

procedure TfrmSysTree.PopulateNetNode;
const
  C1 = '<unavailable>';
var
  S: String;
  {$IFDEF VER120}
  I:DWORD ;
  {$ELSE}
  I: Integer ;
  {$ENDIF}
begin
  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Width := 130;
    Columns.Items[0].Caption := 'Items';
    Columns.Items[1].Caption := 'Value';
  end;

  SetLength(S, MAX_COMPUTERNAME_LENGTH + 1);
  I := MAX_COMPUTERNAME_LENGTH + 1;

  GetComputerName(PChar(S), I);
  RealizeLength(S);

  with ListView1.Items.Add do begin
    Caption := 'Computer name';
    SubItems.Add (S)
  end;

  with ListView1.Items.Add do begin
    Caption := 'Network';
    if bool (GetSystemMetrics (SM_NETWORK)) then
      SubItems.Add('Installed')
    else
      SubItems.Add('Not installed');
  end;

  with ListView1.Items.Add do begin
    with TIniFile.Create('system.ini') do
      try
        Caption := 'Description';
        SubItems.Add(ReadString('boot.description','network.drv', C1));
      finally
        Free
      end
  end;
end;

procedure TfrmSysTree.PopulateLangNode;
  procedure Add(const ACaption, AValue : string);
  begin
    with ListView1.Items.Add do  begin
      Caption := ACaption;
      SubItems.Add (AValue);
    end;
  end;

  procedure AddFmt(const ACaption, AFormat : string; const Value: array of const );
  begin
    Add(ACaption, Format(AFormat, Value));
  end;

  procedure AddInstalled (const Caption : string; B : Bool);
  begin
    if B then
      Add(Caption, 'Installed')
    else
      Add(Caption, 'Not installed');
  end;

  procedure AddEnabled (const Caption : string; B : Bool);
  begin
    if B then
      Add(Caption, 'Enabled')
    else
      Add(Caption, 'Disabled');
  end;

var
  S: String;
begin

  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Width := 130;
    Columns.Items[0].Caption := 'Items';
    Columns.Items[1].Caption := 'Value';
  end;

  SetLength(S,255);

  if SystemParametersInfo(SPI_LANGDRIVER, 0, @S, 0) then begin
    RealizeLength (S);
    Add('Language driver',S);
  end else
    Add('Language driver',C_UNKNOWN);

  AddInstalled('Hebrew/Arabic languages',
    Bool (GetSystemMetrics(SM_MIDEASTENABLED)));

  with SysLocale do begin
    AddEnabled('Far East', Bool (FarEast));
  end;

  Add('Currency string', CurrencyString);
  AddFmt('Currency format', '%d', [CurrencyFormat]);
  AddFmt('Negative currency format', '%d', [NegCurrFormat]);
  Add('Thousand separator', ThousandSeparator);
  Add('Decimal separator', DecimalSeparator);
  AddFmt('Currency decimals', '%d', [CurrencyDecimals]);
  Add('Date separator', DateSeparator);
  Add('Short date format', ShortDateFormat);
  Add('Long date format', LongDateFormat);
  Add('Time separator', TimeSeparator);
  Add('Time AM string', TimeAMString);
  Add('Time PM string', TimePMString);
  Add('Short time format', ShortTimeFormat);
  Add('Long time format', LongTimeFormat);
  with TIniFile.Create ('win.ini') do
  try
    Add('Time Zone is',ReadString ('MAPI 1.0 Time Zone','StandardName',C_UNKNOWN));
  finally
    Free;
  end;

end;

procedure TfrmSysTree.PopulateDiskNode;
begin
  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Caption := 'Items';
    Columns.Items[1].Caption := 'Value';
  end;
  
  DrivesCB.Visible := True;
  GetLogicalDriveList(DrivesCB.Items);

  DrivesCB.ItemIndex := DrivesCB.Items.IndexOf(cPath[1] + ':\');
  FillDiskView;
end;

procedure TfrmSysTree.FillDiskView;
  procedure Add(const ACaption, AValue : string);
  begin
    with ListView1.Items.Add do  begin
      Caption := ACaption;
      SubItems.Add (AValue);
    end;
  end;
var
  Drive : Char;
  DriveType : TDriveType;
  Info : TDriveShellInfo;
  VolumeInfo : TVolumeInformation;
  DiscFreeSpace : TDiscFreeSpace;
begin

  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Width := 130;
    Columns.Items[0].Caption := 'Items';
    Columns.Items[1].Caption := 'Value';
  end;

  Drive := DrivesCB.Items[DrivesCB.ItemIndex][1];
  DriveType := GetDriveType(Drive);

  if DriveType = dtFloppy then begin
    DriveType := FloppyDriveSize(Drive);
    Screen.Cursor := crHourglass
  end;

  StatusPanel.Caption := STATUS_WAIT_MESSAGE;
  StatusPanel.Refresh;
    
  try
    Add('Drive type',DriveNames[DriveType]);
    GetDriveShellInfo(Drive, Info);
    { DriveImage := Info.Image;
    FillInImage(Self);  }
    Add('ShellName',Info.DisplayName);

    while not GetVolumeInformation(Drive, VolumeInfo) do
      if not Retry (Drive) then Abort;

    with VolumeInfo do begin
      if DriveType = dtNetwork then
        Add('Volume label',GetNetworkVolumeName(Drive))
      else
        if VolumeName = '' then
          Add('Volume label','[none]')
        else
          Add('Volume label',VolumeName);

      Add('SerialNumber',FormatVolumeSerialNumber(VolumeSerialNumber));

      Add('Flags','');
      if FileSystemFlags and FS_CASE_IS_PRESERVED <> 0 then
        Add('','FS_CASE_IS_PRESERVED');

      if FileSystemFlags and FS_CASE_SENSITIVE <> 0 then
        Add('','FS_CASE_SENSITIVE');

      if FileSystemFlags and FS_UNICODE_STORED_ON_DISK <> 0 then
        Add('','FS_UNICODE_STORED_ON_DISK');

      if FileSystemFlags and FS_PERSISTENT_ACLS <> 0 then
        Add('','FS_PERSISTENT_ACLS');

      if FileSystemFlags and FS_VOL_IS_COMPRESSED <> 0 then
        Add('','FS_VOL_IS_COMPRESSED')
      else
        if FileSystemFlags and FS_FILE_COMPRESSION <> 0 then
          Add('','FS_FILE_COMPRESSION');

      Add('FileSystem',FileSystemName)
    end;

    while not GetDiscFreeSpace (Drive, DiscFreeSpace) do
      if not Retry (Drive) then Abort;

    with DiscFreeSpace do begin
      Add('SectorsPerCluster', IntToStr(SectorsPerCluster));
      Add('BytesPerSector', IntToStr(BytesPerSector));
      Add('FreeClusters', IntToStr(NumberOfFreeClusters));
      Add('Clusters',IntToStr(TotalNumberOfClusters));
      Add('FreeSpace', IntToStr(FreeSpace));
      Add('DiskSpace', IntToStr(TotalSpace));
    end;
    {ListView1.Selected := ListView1.TopItem;
    ListView1.SetFocus;}
  finally
    Screen.Cursor := crDefault;
    StatusPanel.Caption := '';
  end;
end;

procedure TfrmSysTree.FormResize(Sender: TObject);
begin
  DrivesCB.Left := (ClientWidth - DrivesCB.Width);
  cmdExit.Left := (ClientWidth - cmdExit.Width);
end;

procedure TfrmSysTree.DrivesCBChange(Sender: TObject);
begin
  FillDiskView;
end;

procedure TfrmSysTree.PopulateAppsRunNode;
var
  i,n:Integer;
begin
  Screen.Cursor := crHourglass;
  StatusPanel.Caption := STATUS_WAIT_MESSAGE;
  StatusPanel.Refresh;

  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Width := 130;
    Columns.Items[0].Caption := 'Module Name';
    Columns.Items[1].Caption := 'Process ID';
  end;

  ProcLister := TProcList.Create(Self);
  ProcPaths.Clear;

  try
    with ProcLister do begin
      GetProcessInfo;
      n := ProcessList.Count;
      for i := 0 to n -1 do begin
        with ListView1.Items.Add do begin
          Caption := ProcessList[i];
          SubItems.Add(IntToHex(GetP_ID(i),8));
        end;
        ProcPaths.Add(GetP_ExePath(i));
      end;
    end;
  finally
    ProcLister.Free;
    Screen.Cursor := crDefault
  end;

  StatusPanel.Caption := '';

end;

procedure TfrmSysTree.PopulateModemNode;
begin
  ClearListViewHeaders;
end;

procedure TfrmSysTree.PopulatePrintNode;
var
  iCount,i:Integer;
  S,P:String;
begin

  ClearListViewHeaders;

  with ListView1 do begin
    Items.Clear ;
    Columns.Items[0].Width := 10;    
    Columns.Items[0].Caption := '';
    Columns.Items[1].Caption := 'Print device(s)';
  end;

  iCount := Printer.Printers.Count;
  if iCount = 0 then  exit;

  S := printer.Printers[Printer.PrinterIndex];
  for i := 0 to iCount -1 do begin
    P := Printer.Printers[i];
    with ListView1.Items.Add do  begin
      SubItems.Add (P);
      if P = S then
        Caption := '*';
    end;
  end;
end;

procedure TfrmSysTree.cmdExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmSysTree.TreeView1DblClick(Sender: TObject);
begin
  with (Sender as TTreeView) do begin
    if Selected.Level = 0 then
      if Selected.Index >0 then
        if Integer(Selected.Data) = 20 then
          ShowAboutScreen;
  end;
end;

procedure TfrmSysTree.ShowAboutScreen;
begin
  ShowMessage('As you can see, this is not finished');
end;

{ Insure the first item is highlighted, default is nothing highlighted}
procedure TfrmSysTree.ListView1Enter(Sender: TObject);
begin
  with TListView(Sender) do begin
    if Items.Count > 0 then
      Selected := TopItem;
  end;
end;

procedure TfrmSysTree.cmdPrintInformationClick(Sender: TObject);
begin
  { send report to printer }
end;

procedure TfrmSysTree.cmdDiskReportClick(Sender: TObject);
begin
  { send report to disk file of users choice }
end;

procedure TfrmSysTree.FormCreate(Sender: TObject);
begin
  ProcPaths := TStringList.Create;
end;

procedure TfrmSysTree.ListView1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  z:Integer;
begin
  if not (ssRight in Shift) then
    exit;

  with (Sender as TListView) do begin
    if SelCount = 0 then exit;
    z := Selected.Index;
    if Columns.Items[0].Caption = 'Module Name' then begin
      ItemProp.DisplayPropertiesDialog(ProcPaths.Strings[z],Self.Handle)
    end;
  end;

end;

procedure TfrmSysTree.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ProcPaths.Free;
end;

end.
